<?php

/**
 * System settings page.
 */
function background_batch_settings_form() {
  $form = array();
  $form['background_batch_cleanup'] = array(
    '#type' => 'textfield',
    '#default_value' => variable_get('background_batch_cleanup', BACKGROUND_BATCH_CLEANUP),
    '#title' => 'Cleanup',
    '#description' => t('Time in seconds before finished jobs are removed from the database'),
  );
  return system_settings_form($form);
}

/**
 * Overview of current and recent batch jobs.
 */
function background_batch_overview_page() {
  $sql = "
SELECT b.bid, p.progress, p.message
FROM batch b
JOIN progress p
ON p.name = CONCAT('background_batch:', b.bid)
";
  $result = db_query($sql);
  while ($row = db_fetch_object($result)) {
    $data[] = array(
      l($row->bid, 'batch', array('query' => 'op=start&id=' . $row->bid)),
      ($row->progress * 100) . '%',
      $row->message,
    );
  }
  $header = array('Batch ID', 'Progress', 'Message');
  return theme('table', $header, $data);
}

/**
 * State-based dispatcher for the batch processing page.
 */
function background_batch_progress_page() {
  $batch =& batch_get();

  $id = isset($_REQUEST['id']) ? $_REQUEST['id'] : FALSE;

  // Retrieve the current state of batch from db.
  if ($id && $data = db_result(db_query("SELECT batch FROM {batch} WHERE bid = %d AND token = '%s'", $id, drupal_get_token($id)))) {
    $batch = unserialize($data);
  }
  else {
    $batch = FALSE;
  }

  $op = isset($_REQUEST['op']) ? $_REQUEST['op'] : '';
  switch ($op) {
    case 'start':
      if (!$batch) {
        return drupal_not_found();
      }
      // If batch is already in progress, goto to the status page instead of starting it.
      $progress = progress_get_progress('background_batch:' . $id);
      if (!$progress) {
        // @todo Lock Process -> Init Progress -> Execute Process
        //       instead of
        //       Init Progress -> Lock Process -> Execute Process
        //       for easier cleanup of progress.
        progress_initialize_progress('background_batch:' . $id, 'Initialized');
        if (!background_process_start('_background_batch_process', $id)) {
          // Error!!!
          return drupal_not_found();
        }
      }
      return _background_batch_progress_page_js();
    case 'do':
      return _background_batch_do();
    case 'finished':
      return _background_batch_finished_page();
  }
}

/**
 * Batch processing page with JavaScript support.
 */
function _background_batch_progress_page_js() {
  require_once('includes/batch.inc');

  $batch = batch_get();

  // The first batch set gets to set the page title
  // and the initialization and error messages.
  $current_set = _batch_current_set();
  drupal_set_title($current_set['title']);
  drupal_add_js('misc/progress.js', 'core', 'header', FALSE, FALSE);

  $url = url($batch['url'], array('query' => array('id' => $batch['id'])));
  $js_setting = array(
    'batch' => array(
      'errorMessage' => $current_set['error_message'] .'<br/>'. $batch['error_message'],
      'initMessage' => $current_set['init_message'],
      'uri' => $url,
    ),
  );
  drupal_add_js($js_setting, 'setting');
  drupal_add_js('misc/batch.js', 'core', 'header', FALSE, FALSE);

  $output = '<div id="progress"></div>';
  return $output;
}

/**
 * Do one pass of execution and inform back the browser about progression
 * (used for JavaScript-mode only).
 */
function _background_batch_do() {
  // HTTP POST required
  if ($_SERVER['REQUEST_METHOD'] != 'POST') {
    drupal_set_message(t('HTTP POST is required.'), 'error');
    drupal_set_title(t('Error'));
    return '';
  }

  $id = isset($_REQUEST['id']) ? $_REQUEST['id'] : FALSE;

  // Get progress
  $progress = progress_get_progress('background_batch:' . $id);
  $percentage = $progress->progress * 100;
  $message = $progress->message;

  usleep(100000);
  drupal_json(array('status' => TRUE, 'percentage' => $percentage, 'message' => $message));
}

/**
 * Resolve page redirection for a finished batch job.
 */
function _background_batch_finished_page() {
  $batch =& batch_get();
  $_batch = $batch;
  $batch = NULL;

  // Redirect if needed.
  if ($_batch['progressive']) {
    // Put back the 'destination' that was saved in batch_process().
    if (isset($_batch['destination'])) {
      $_REQUEST['destination'] = $_batch['destination'];
    }

    // Use $_batch['form_state']['redirect'], or $_batch['redirect'],
    // or $_batch['source_page'].
    if (isset($_batch['form_state']['redirect'])) {
      $redirect = $_batch['form_state']['redirect'];
    }
    elseif (isset($_batch['redirect'])) {
      $redirect = $_batch['redirect'];
    }
    else {
      $redirect = $_batch['source_page'];
    }

    // Let drupal_redirect_form handle redirection logic.
    $form = isset($batch['form']) ? $batch['form'] : array();
    if (empty($_batch['form_state']['rebuild']) && empty($_batch['form_state']['storage'])) {
      drupal_redirect_form($form, $redirect);
    }

    // We get here if $form['#redirect'] was FALSE, or if the form is a
    // multi-step form. We save the final $form_state value to be retrieved
    // by drupal_get_form, and we redirect to the originating page.
    $_SESSION['batch_form_state'] = $_batch['form_state'];
    drupal_goto($_batch['source_page']);
  }
}
